@@ -1,6 +1,6 @@
 /* libpq.c
 
-   Copyright (C) 2003-2015 Thien-Thi Nguyen
+   Copyright (C) 2003-2015, 2017 Thien-Thi Nguyen
    Portions Copyright (C) 1999, 2000 Ian Grant
 
    This file is part of Guile-PG.
@@ -665,44 +665,6 @@ lob_input_waiting_p (UNUSED SCM port)
   return 1;
 }
 
-/* Define this here because ‘pg-lo-read’ is going away RSN; there's
-   no point in (even eventually) bubbling this up to Guile-BAUX.  */
-#if GI_LEVEL (1, 9)
-#define TAKE_STR  scm_take_locale_stringn
-#else
-#define TAKE_STR  scm_take_str
-#endif
-
-PRIMPROC
-(pg_lo_read, "pg-lo-read", 3, 0, 0,
- (SCM siz, SCM num, SCM port),
- doc: /***********
-@zonkable{2015-12-31,procedure}
-
-Read @var{num} objects each of length @var{siz} from @var{port}.
-Return a string containing the data read from the port or
-@code{#f} if an error occurred.  */)
-{
-#define FUNC_NAME s_pg_lo_read
-  char *stage, *wp;
-  int csiz, cnum, len, c;
-
-  VALIDATE_NNINT_COPY (1, siz, csiz);
-  VALIDATE_NNINT_COPY (2, num, cnum);
-  ASSERT_PORT (3, port, OPINLOBPORTP);
-
-  if (0 > (len = csiz * cnum)
-      || (MAX_NEWSTRING_LENGTH < len)
-      || (! (wp = stage = (char *) malloc (1 + len))))
-    RETURN_FALSE ();
-  while (len-- && (EOF != (c = scm_getc (port))))
-    *wp++ = c;
-  *wp = '\0';
-  /* TODO: Cascade zonking to ‘TAKE_STR’ definition, too.  */
-  return TAKE_STR (stage, wp - stage);
-#undef FUNC_NAME
-}
-
 static int
 lob_close (SCM port)
 {
