@@ -1,6 +1,6 @@
 ;;; postgres-meta.scm --- Methods for understanding PostgreSQL data structures
 
-;; Copyright (C) 2002-2009, 2011, 2014, 2015 Thien-Thi Nguyen
+;; Copyright (C) 2002-2009, 2011, 2014, 2015, 2017, 2020 Thien-Thi Nguyen
 ;;
 ;; This file is part of Guile-PG.
 ;;
@@ -36,13 +36,22 @@
   #:use-module ((database postgres-types)
                 #:select (type-registered?
                           define-db-col-type
-                          define-db-col-type-array-variant))
+                          register-array-variant))
   #:use-module ((database postgres-resx)
                 #:select (result-field->object-list))
   #:use-module ((database postgres-table)
                 #:select (pgtable-manager
                           compile-outspec))
-  #:autoload (srfi srfi-13) (string-trim-both)
+  #:use-module ((srfi srfi-13)
+                #:select (string-index
+                          string-prefix?
+                          substring/shared
+                          string-concatenate-reverse
+                          string-skip
+                          string-take
+                          string-map
+                          (string-trim-both . stb)))
+  #:use-module ((ice-9 rdelim) #:select (read-line))
   #:autoload (ice-9 popen) (open-input-pipe))
 
 (define (fs s . args)
@@ -589,25 +598,22 @@
 
     (define (read-options)              ; => eof object, or list
 
-      (define (stb s)
-        (string-trim-both s))
-
       (define (line-remainder)
-        (let drain ((c (read-char p)) (acc '()))
-          (if (char=? #\newline c)
-              (stb (list->string (reverse! acc)))
-              (drain (read-char p)
-                     (cons (if (char=? #\, c) #\space c) acc)))))
+        (stb (string-map (lambda (c)
+                           (case c
+                             ((#\,) #\space)
+                             (else c)))
+                         (read-line p))))
 
       (define (eol-if-null-string s)
         (and (string-null? s) '()))
 
       (let ((opts (line-remainder)))
         (cond ((eol-if-null-string opts))
-              ((and (char=? #\( (string-ref opts 0))
+              ((and (string-prefix? "(" opts)
                     (string-index opts #\)))
                => (lambda (cut)
-                    (let ((rest (stb (substring opts (1+ cut)))))
+                    (let ((rest (stb (substring/shared opts (1+ cut)))))
                       (cons (with-input-from-string opts read)
                             (or (eol-if-null-string rest)
                                 (list rest))))))
@@ -635,15 +641,14 @@
 ;;
 (define (check-type/elaborate type)
   (let* ((s (symbol->string type))
-         (n (string-index s #\[))
-         (base (string->symbol (if n (substring s 0 n) s))))
+         (n (string-skip s #\*))
+         (base (if (zero? n)
+                   type
+                   (string->symbol (substring s n)))))
     (and (type-registered? base)
-         (or (not n)
-             (let ((array-variant type))
-               (or (type-registered? array-variant)
-                   (begin
-                     (define-db-col-type-array-variant array-variant base)
-                     array-variant)))))))
+         (or (zero? n)
+             (or (type-registered? type)
+                 (register-array-variant n base))))))
 
 ;; For table @var{table-name}, check @var{types} (list of symbols) with
 ;; @code{check-type/elaborate} and signal error for those types that do not
