@@ -1,6 +1,6 @@
 ;;; basic.scm
 
-;; Copyright (C) 2002-2011, 2014 Thien-Thi Nguyen
+;; Copyright (C) 2002-2011, 2014, 2017 Thien-Thi Nguyen
 ;; Portions Copyright (C) 1999, 2000 Ian Grant
 ;;
 ;; This file is part of Guile-PG.
@@ -25,7 +25,8 @@
 
 (reset-all-tests!)
 
-(use-modules (database postgres))
+(use-modules (database postgres)
+             (srfi srfi-13))
 
 ;; We use one connection for all the tests.
 
@@ -90,37 +91,17 @@
               (iter (- n 1))))))
   (iter n))
 
-(define (as-string obj)
-  (cond ((string? obj)    obj)
-        ((char? obj)      (make-string 1 obj))
-        ((symbol? obj)    (symbol->string obj))
-        ((number? obj)    (number->string obj))
-        ;; this infinite loops -ttn
-        ;; ((procedure? obj) (as-string (obj)))
-        ((pair? obj) (interpolate-list "(" " " ")"
-                                       (lambda (x) (as-string x))
-                                       obj))
-        (else (call-with-output-string (lambda (port) (write obj port))))))
-
-(define (interpolate-list afore twixt aft proc lst)
-  (define (iter lst)
-    (let ((elt (car lst))
-          (tail (cdr lst)))
-      (string-append (proc elt)
-                     (if (not (null? tail))
-                         (string-append (as-string twixt) (iter tail))
-                         (as-string aft)))))
-  (string-append (as-string afore) (iter lst)))
-
 (define (make-table! tablename fields)
   (cexec (string-append
           "CREATE TABLE " tablename " "
-          (interpolate-list "(" ", " ")"
-                            (lambda (p)
+          "("
+          (string-join (map (lambda (p)
                               (string-append
-                               (as-string (car p)) " "
-                               (as-string (cdr p))))
+                               (car p) " "
+                               (cdr p)))
                             fields)
+                       ", ")
+          ")"
           ;; starting w/ PostgreSQL 8.1, this needs to be explicit.
           " WITH OIDS")))
 
